from os import rename, stat, remove, fsync, posix_fadvise, \
        POSIX_FADV_DONTNEED, chmod, close, stat
from shutil import move
from os.path import join
from tempfile import mkstemp

from xml.etree import ElementTree as ET
from xml.dom import minidom

import hashlib, zlib

from json import dump

from subprocess import check_output, check_call, CalledProcessError, \
        DEVNULL, STDOUT
from collections import OrderedDict

import xmltodict

from .const import PB_ITEM, PB_TASK, PB_TMP


def parse_item_metadata(path):
    tree = ET.parse(path)
    root = tree.getroot()
    if root.tag != 'metadata':
        raise ValueError('No \'metadata\' in _meta.xml')
    md = {}

    for elem in root:
        if elem.text is None:
            continue
        if elem.tag in md:
            if isinstance(md[elem.tag], list):
                md[elem.tag].append(elem.text)
            else:
                md[elem.tag] = [md[elem.tag], elem.text]
        else:
            md[elem.tag] = elem.text

    return md


def load_item_metadata(identifier):
    """
    Returns the metadata of an item as a (modifiable) dictionary.

    Args:

    * identifier (``str``): Identifier of the item

    Returns:

    * item metadata (``dict``)

    Example usage::

        from derivermodule.task import get_task_info
        from derivermodule.metadata import load_item_metadata

        info = get_task_info()
        identifier = info['identifier']
        metadata = load_item_metadata(identifier)
    """
    path = open(join(PB_ITEM, '%s_meta.xml' % identifier))
    return parse_item_metadata(path)


def metadata_to_metaxml(metadata):
    meta_root = ET.Element('metadata')

    for key, value in metadata.items():
        if isinstance(value, list):
            for val in value:
                elem = ET.SubElement(meta_root, key)
                elem.text = val
        elif isinstance(value, str):
            elem = ET.SubElement(meta_root, key)
            elem.text = value
        else:
            raise ValueError('Unknown value type: %s' % type(value))

    # TODO: I think we can get rid of minidom here
    result = minidom.parseString(
        ET.tostring(meta_root, encoding='UTF-8')
    ).toprettyxml(
        encoding='UTF-8',
        indent=' ' * 2
    )

    xml = check_output(['xmllint', '--format', '-'], input=result)

    return xml.decode('utf-8')


def write_item_metadata(identifier, metadata):
    """
    Write the (changed) item metadata dictionary to disk.
    This is required if changes made to the `metadata` are to persist.
    The `metadata` should be loaded using `load_item_metadata`.

    Args:

    * identifier (``str``): Identifier of the item
    * metadata (``dict``): Metadata, loaded by `load_item_metadata` and
                           potentially modified.

    Returns:

    * Nothing
    """
    metaxml = metadata_to_metaxml(metadata)
    fp, tmp_path = mkstemp(suffix='.xml', dir=PB_TMP, text=True)
    close(fp)

    # We write the XML to a temporary file, flush it, sync it to disk, and
    # finally tell Linux to drop it from the cache, to prevent the disk from
    # silently corrupted the file before xmllint performs the final validation.
    # Alternatively, we could write it to disk and read it from disk with
    # O_DIRECT, and pass that to xmllint's stdin.
    fp = open(tmp_path, 'w+')
    fp.write(metaxml)
    fp.flush()
    fsync(fp.fileno())
    posix_fadvise(fp.fileno(), 0, 0, POSIX_FADV_DONTNEED)
    fp.close()

    try:
        check_call(['xmllint', '--format', tmp_path], stdout=DEVNULL, stderr=STDOUT)
    except CalledProcessError:
        remove(tmp_path)
        raise

    meta_xml_path = join(PB_ITEM, '%s_meta.xml' % identifier)
    st = stat(meta_xml_path)
    chmod(tmp_path, st.st_mode)
    move(tmp_path, meta_xml_path)


def load_files_metadata(identifier):
    """
    Returns the file-level metadata of an item as an opaque (not to be directly
    modified) object. Use `lookup_file_metadata` to get a reference to the
    metadata of a specific file.

    There is no way to save changes made to this structure, so users have to
    treat the result as read-only.

    Args:

    * identifier (``str``): Identifier of the item

    Returns:

    * object containing the file-level metadata of an item.
    """
    path = open(join(PB_ITEM, '%s_files.xml' % identifier))
    data = path.read()
    path.close()
    return xmltodict.parse(data, strip_whitespace=False)


def write_files_metadata(files_metadata):
    """
    Write additional metadata for a set of files.
    This function cannot add metadata to arbitrary files, only to the target
    file and any extra targets specified in
    ``derivermodule.task.write_extra_targets``. It also cannot delete metadata
    values, nor does it have to: any files that are created by this module will
    have no metadata associated with it, so there will be nothing to delete.

    Metadata values can be of type ``str`` or ``list (of str)``.

    Args:

    * files_metadata: a dictionary where the key is a canonical item filename
                      (see ``derivermodule.files.canonical_item_filename``)
                      and the value is a key-value metadata pairs.

    Returns:

    * Nothing

    Example:

    >>> from derivermodule.files import canonical_item_filename
    >>> md = {'foo_module_version': '0.0.1'}
    >>> write_files_metadata({canonical_item_filename(target_file): md})
    """
    files_metadata_file = join(PB_TASK, 'extra_metadata.json')
    fp = open(files_metadata_file, 'w+')
    dump(files_metadata, fp)
    fp.close()


## TODO: Perform a *LOT* of testing on this
#def write_files_metadata(identifier, metadata):
#    """
#    Writes the (changed) file level metadata dictionary to disk.
#    This is required if changes made to the `metadata` are to persist.
#    The `metadata` should be loaded using `load_files_metadata` and not changed
#    directly, only by calling `lookup_file_metadata` and changing the values in
#    the result of that function.
#
#    Args:
#
#    * identifier (``str``): Identifier of the item
#    * metadata (``dict``): Metadata, loaded by `load_files_metadata` and
#                           potentially modified.
#
#    Returns:
#
#    * Nothing
#    """
#    # TODO: Sort by @name entries, alphabetically?
#    result = xmltodict.unparse(metadata, pretty=True).encode('utf-8')
#    metafilesxml = check_output(['xmllint', '--format', '-'], input=result)
#    fp = open(join(PB_ITEM, '%s_files.xml_tmp' % identifier), 'wb+')
#    fp.write(metafilesxml)
#    fp.close()
#    rename(join(PB_ITEM, '%s_files.xml_tmp' % identifier), join(PB_ITEM, '%s_files.xml' % identifier))


def lookup_file_metadata(files_metadata, filename):
    """
    Fetch file-level metadata for a specific file, as a dictionary
    (``collections.OrderedDict``)

    Args:

    * files_metadata: files metadata as returned by `load_files_metadata`.
    * filename (``str``): filename/path canonical (relative) to the item

    Returns:

    File-level metadata if it exists (``dict``) or ``None``.
    """
    # XXX: in doc, mention that changing the properties here should reflect in
    # final written file
    file_list = files_metadata['files']['file']

    for file_info in file_list:
        if file_info['@name'] == filename:
            return file_info

    return None


def lookup_file_derivative_path(files_metadata, filename):
    """
    Lookup the derivation path of a given file and return it as a list.

    For example, a $foo_page_numbers.json file is typically derived from a
    $foo_djvu.xml, which is derived from a $foo_hocr.html, which is derived from
    a $foo_chocr.html.gz file, which can be derived from a $foo_jp2.zip file,
    which could be derived from a $foo.pdf file.

    If the derivative path is requested for the 'foo_page_numbers.json' file,
    the return value would be as follows:

    ['foo_page_numbers.json',
     'foo_djvu.xml',
     'foo_hocr.html',
     'foo_chocr.html.gz',
     'foo_jp2.zip',
     'foo.pdf']

    Args:

    * files_metadata: files metadata as returned by `load_files_metadata`.
    * filename (``str``): filename/path canonical (relative) to the item

    Returns:

    Derivative path as a list of filenames in the item.

    May raise a ValueError is the path is abrupted suddenly
    """
    count = 0
    path = []

    md = lookup_file_metadata(files_metadata, filename)

    while True:
        count += 1
        path.append(md['@name'])

        deriv = md['@source']
        if deriv == 'original':
            return path

        md = lookup_file_metadata(files_metadata, md['original'])

        if count > 1000:
            raise ValueError('Refusing to recursive beyond 1000 files, corrupted files metadata?')

    raise ValueError('Search did not end in a source original file')


##: Indicates that a file is an original file (used with create_file_metadata)
#SOURCE_ORIGINAL = 'original'
#
##: Indicates that a file is a derivative file (used with create_file_metadata)
#SOURCE_DERIVATIVE = 'derivative'
#
#def create_file_metadata(files_metadata, filename, source=SOURCE_ORIGINAL, fileformat=None):
#    if lookup_file_metadata(files_metadata, filename) is not None:
#        raise ValueError('%s already exists in files_metadata' % filename)
#
#    # XXX: in doc, mention that one should be really careful with this
#    if source not in (SOURCE_ORIGINAL, SOURCE_DERIVATIVE):
#        raise ValueError('Invalid source type.')
#
#    if fileformat is None:
#        raise ValueError('Please specify a valid fileformat')
#
#    entry = OrderedDict([('@name', filename),
#             ('@source', source),
#             ('format', fileformat)])
#
#    return entry
#
#
#def append_file_metadata(files_metadata, file_entry):
#    # TODO: for doc, requires file to exist on disk in PB_ITEM
#
#    #md = _calculate_file_metadata(join(PB_ITEM, file_entry['@name']))
#    #file_entry.update(md)
#    files_metadata['files']['file'].append(file_entry)
#
#
#def _calculate_file_metadata(filename):
#    data = {}
#    data.update(_calc_hashes(filename))
#
#    stat_data = stat(filename)
#    data['mtime'] = stat_data.st_mtime
#    data['size'] = stat_data.st_size
#
#    return data
#
#
#def _calc_hashes(filename):
#    fd = open(filename, 'rb')
#    crc32 = 0
#    md5 = hashlib.md5()
#    sha1 = hashlib.sha1()
#
#    while True:
#        s = fd.read(65536)
#        if not s:
#            break
#        crc32 = zlib.crc32(s, crc32)
#        md5.update(s)
#        sha1.update(s)
#
#    fd.close()
#
#    data = {'crc32': ('%08X' % crc32).lower(),
#            'md5': md5.hexdigest(),
#            'sha1': sha1.hexdigest() }
#
#    return data



import tempfile
import zipfile
from subprocess import check_call
from pathlib import Path

import xmltodict

from .const import PB_ITEM, PB_TMP
from .logger import logger


def get_scandata_xml(identifier, source_file):
    """
    Parses the scandata.xml file for a given identifier and source_file.

    Args:

    * identifier (``str``): Identifier of the item
    * source_file (``str``): sourceFile to be operated on

    Returns:

    * Path to the scandata (``str``) or None
    """
    item_dir = Path(PB_ITEM)

    zip_path = item_dir/'scandata.zip'
    raw_xml_path = item_dir/'scandata.xml'
    if source_file:
        if source_file.endswith('page_numbers.json'):
            source_xml_path = Path(f'{source_file[:-18]}_scandata.xml')
        else:
            source_xml_path = Path(f'{source_file.rsplit("_", 1)[0]}_scandata.xml')

    if source_xml_path.exists():
        return str(source_xml_path)
    elif raw_xml_path.exists():
        return str(raw_xml_path)
    elif zip_path.exists():
        try:
            # XXX: We don't clean up the temporary directory when we're done,
            # since the Docker container will remove all files upon completion.
            directory = tempfile.mkdtemp(dir=PB_TMP)
            xml_path = Path(directory)/'scandata.xml'

            zf = zipfile.ZipFile(zip_path)
            sf = zf.open('scandata.xml')
            f = open(xml_path, 'wb+')
            f.write(sf.read())
            f.close()
            sf.close()
            zf.close()

            assert(xml_path.exists())
        except Exception as e:
            logger.warning('Unable to extract scandata.xml from scandata.zip')
            logger.exception(e)
            return None

        return str(xml_path)

    logger.warning(f'Unable to find scandata file')
    return None


def scandata_parse(scandata_path):
    """
    Parse scandata.xml to native Python format

    Args:

    * scandata_path (``str``): Path to the scandata

    Returns:

    * Scandata as dictionary
    """
    scandata = xmltodict.parse(open(scandata_path, 'rb').read())
    return scandata


def scandata_get_page_count(scandata):
    """
    Get the number of page elements in a parsed scandata object

    Args:

    * scandata (``dict``): Scandata as returned by `scandata_parse`.

    Returns:

    * The number of page elements (``int``)
    """
    pages = scandata.get('book', {}).get('pageData', {}).get('page', [])
    if not isinstance(pages, list):
        pages = [pages]

    return len(pages)


def scandata_get_skip_pages(scandata):
    """
    Returns a list of indexes of pages in scandata.xml that have
    addToAccessFormats = false

    Args:

    * scandata: Parsed scandata as returned by scandata_parse

    Returns:

    * Indexes of pages that should not added to access formats
      (``list of int``)
    """
    skip = []

    pages = scandata['book']['pageData']['page']

    # If there is just one page, pages is not a list.
    if not isinstance(pages, list):
        pages = [pages]

    for idx in range(len(pages)):
        try:
            add_to_access_format = pages[idx]['addToAccessFormats']
            if add_to_access_format == 'false':
                skip.append(idx)
        except KeyError:
            pass

    return skip

def scandata_get_pagetype_pages(scandata, page_type):
    """
    Returns a list of indexes of pages in scandata.xml that have
    a specific page_type - case insensitive

    Args:

    * scandata: Parsed scandata as returned by scandata_parse
    * page_type: Page type as string, e.g. 'Cover', 'Normal', etc (full list not provided)

    Returns:

    * Indexes of pages that match a specific page type
      (``list of int``)
    """
    page_type_match = []

    pages = scandata['book']['pageData']['page']

    # If there is just one page, pages is not a list.
    if not isinstance(pages, list):
        pages = [pages]

    page_type = page_type.lower()

    for idx in range(len(pages)):
        try:
            if pages[idx]['pageType'].lower() == page_type:
                page_type_match.append(idx)
        except KeyError:
            pass

    return page_type_match



import pathlib

from .const import PB_ITEM

def canonical_item_filename(filepath):
    """
    Normalises an absolute path to a file in the container to a canonical path.
    For example '/item/test_item/test_directory/test_file_hocr.html.gz' would be
    turned into 'test_item/test_directory/test_file_hocr.html.gz'.

    Args:

    * filepath (``str``): Absolute path to the file within the container

    Returns:

    * canonical path (``str``)
    """
    root = pathlib.Path(PB_ITEM)
    item_path = pathlib.Path(filepath)
    rel = item_path.relative_to(root)

    return str(rel)



import logging
import logging.config

def set_level_debug():
    logger.setLevel(logging.DEBUG)
    ch.setLevel(logging.DEBUG)

def set_level_info():
    logger.setLevel(logging.INFO)
    ch.setLevel(logging.INFO)

def set_level_warning():
    logger.setLevel(logging.WARNING)
    ch.setLevel(logging.WARNING)

def set_level_error():
    logger.setLevel(logging.ERROR)
    ch.setLevel(logging.ERROR)

def set_level_critical():
    logger.setLevel(logging.CRITICAL)
    ch.setLevel(logging.CRITICAL)

def get_logger(logger_name):
    """
    Returns an instance of ``logging.Logger`` with a specific log format and
    logging level.
    """
    DEFAULT_LOGGING_LEVEL = logging.INFO
    LOG_FORMAT = '%(asctime)s %(levelname)-8s %(message)s'

    logger = logging.getLogger(logger_name)
    logger.setLevel(DEFAULT_LOGGING_LEVEL)
    ch = logging.StreamHandler()
    ch.setLevel(DEFAULT_LOGGING_LEVEL)
    formatter = logging.Formatter(LOG_FORMAT)
    ch.setFormatter(formatter)
    logger.addHandler(ch)

    return logger

#: Default logger with name ``'derivermodule'``
logger = get_logger('derivermodule')



from pathlib import Path
from os.path import splitext

from subprocess import check_call
from time import time

import zipfile
import tarfile


from .logger import logger

from .metadata import lookup_file_metadata, lookup_file_derivative_path
from .files import canonical_item_filename
from .const import PB_ITEM

IMAGE_TYPES = ['jp2', 'jpg', 'tif']
ARCHIVE_TYPES = ['zip', 'tar']

def get_imagestack_info_from_format(fmt):
    """
    Attempt to detect if a given format is an image stack format

    Return the image stack format info when the format is an imagestack,
    otherwise None.

    Args:

    * fmt (``str``): format
    """
    if fmt == 'Single Page Processed JP2 ZIP':
        return {'image_type': 'jp2', 'archive_type': 'zip'}
    if fmt == 'Single Page Processed JP2 Tar':
        return {'image_type': 'jp2', 'archive_type': 'tar'}
    if fmt == 'Single Page Processed JPEG ZIP':
        return {'image_type': 'jpg', 'archive_type': 'zip'}
    if fmt == 'Single Page Processed JPEG Tar':
        return {'image_type': 'jpg', 'archive_type': 'tar'}
    if fmt == 'Single Page Processed TIFF ZIP':
        return {'image_type': 'tif', 'archive_type': 'zip'}
    if fmt == 'Single Page Processed TIFF Tar':
        return {'image_type': 'tif', 'archive_type': 'tar'}
    return None


def get_imagestack_info(task_info):
    """
    Reads the source_format from the task information (parse with
    task.get_task_info).

    Returns a dictionary with the following information:

    >>> { 'image_type': 'jp2' or 'jpg' or 'tif', 'archive_type': 'zip' or 'tar'. }
    """
    source_format = task_info['sourceFormat']
    info = get_imagestack_info_from_format(source_format)

    if info is None:
        raise ValueError('Unhandled imagestack format: %s' % source_format)

    return info


def get_imagestack_info_for_file(filename, files_metadata):
    """
    Reads the filename format from the files metadata and returns the imagestack
    info

    Returns a dictionary with the following information:

    >>> { 'image_type': 'jp2' or 'jpg' or 'tif', 'archive_type': 'zip' or 'tar'. }
    """
    file_md = lookup_file_metadata(files_metadata, filename)
    if file_md is None:
        raise ValueError('Could not find metadata for file: %s' % filename)

    info = None
    source_format = file_md['format']
    info = get_imagestack_info_from_format(source_format)

    if info is None:
        raise ValueError('Unhandled imagestack format: %s' % source_format)

    return info


def find_imagestack_for_prefix(basefilename, files_metadata):
    """
    Probe for an imagestack in an item give the file base filename. (i.e.
    everything before _jp2.zip, _hocr.html, _djvu.xml, etc)

    Args:

    * basefilename (str): Base filename
    * files_metadata: files metadata as returned by `load_files_metadata`.

    Returns: imagestack info (per `get_imagestack_info_for_file`) or None
    """
    for image_type in IMAGE_TYPES:
        for archive_type in ARCHIVE_TYPES:
            path = basefilename + '_%s.%s' % (image_type, archive_type)
            try:
                return get_imagestack_info_for_file(path, files_metadata)
            except ValueError:
                pass

    return None


def get_source_image_stack_for_file(source_file, files_metadata):
    """
    Find the source image stack for a given source_file, by searching the files
    metadata derivative tree.

    Args:

    * source_file (``str``): canonical filename
    * files_metadata: files metadata as returned by `load_files_metadata`.

    Returns: ``str`` (the filename of the image stack) or ``None``.
    """
    source_file = source_file
    file_derivative_path = lookup_file_derivative_path(files_metadata, source_file)

    for filename in file_derivative_path:
        file_md = lookup_file_metadata(files_metadata, filename)
        file_format = file_md['format']

        imagestack_info = get_imagestack_info_from_format(file_format)
        if imagestack_info is not None:
            return filename

    return None


def unpack_and_validate_imagestack(imagestack_path, imagestack_info, dst):
    """Unpack and validate an imagestack

    An imagestack is valid if it contains at least one directory that contains
    at least one image of the expected image type.

    Unless you need all the images written to disk (which could be slow and take
    up space), it would be better to look at the iterate_imagestack and
    validate_imagestack methods in this module.

    Args:

    * imagestack_path (``str``): The imagestack archive path
    * imagestack_info (``dict``)::

        >>> {'archive_type': ..., 'image_type': ...}

    * dst (``str``): Destination directory for the unpacked imagestack

    Returns:

    * ``(str, int)``: Tuple containing the path to the unpacked image directory
                      and the image count.
    """
    logger.info('Unpacking image stack.')
    start_time = time()
    if imagestack_info['archive_type'] == 'tar':
        check_call(['tar', '-xf', imagestack_path, '-C', dst])
    elif imagestack_info['archive_type'] == 'zip':
        check_call(['unzip', '-qq', '-o', imagestack_path, '-d', dst])
    else:
        raise ValueError('Cannot extract archive_type %s' % imagestack_info['archive_type'])
    logger.info('Unpacking image stack took %f seconds', time() - start_time)

    for f in Path(dst).iterdir():
        if f.name.endswith(f'_{imagestack_info["image_type"]}'):
            img_dir = f
            break
    else:
        raise Exception('Unable to locate image directory in imagestack.')

    image_count = 0
    for img_path in img_dir.iterdir():
        if img_path.suffix[1:] == imagestack_info['image_type']:
            image_count += 1

    if image_count == 0:
        raise Exception('Imagestack contains no valid images.')

    return str(img_dir), image_count


def iterate_imagestack(imagestack_path, imagestack_info, sort=True):
    """Unpack and validate an imagestack

    An imagestack is valid if it contains at least one directory that contains
    at least one image of the expected image type. All operations are done
    without writing to the disk.

    Args:

    * imagestack_path (``str``): The imagestack archive path
    * imagestack_info (``dict``)::

        >>> {'archive_type': ..., 'image_type': ...}

    * sort (``bool``): optional, sort the files by name (sort=False should be
                       faster if you don't care about the order)

    Returns:

    * Iterator of (filename, file handle)
    """
    if imagestack_info['archive_type'] == 'tar':
        tf = tarfile.open(imagestack_path)

        members = tf.getmembers()
        if sort:
            members = sorted(members, key=lambda x: x.name)

        for idx, img in enumerate(members):
            if not img.isfile():
                continue

            ci = tf.extractfile(img)

            yield img.name, ci

            ci.close()

        tf.close()

    elif imagestack_info['archive_type'] == 'zip':
        zf = zipfile.ZipFile(imagestack_path)
        namelist = zf.namelist()

        if sort:
            namelist = sorted(namelist)

        for idx, img in enumerate(namelist):
            info = zf.getinfo(img)
            if info.is_dir():
                continue
            ci = zf.open(img)

            yield info.filename, ci

            ci.close()

        zf.close()
    else:
        raise ValueError('Cannot extract archive_type %s' % imagestack_info['archive_type'])

    return


def validate_imagestack(imagestack_path, imagestack_info):
    """
    Validate an imagestack given a path. An imagestack is valid if it contains
    at least image with a supported image type. All operations are done without
    writing to the disk.

    Args:

    * imagestack_path (``str``): The imagestack archive path
    * imagestack_info (``dict``)::

        >>> {'archive_type': ..., 'image_type': ...}

    If the method does not raise an Exception, the image stack is valid.
    """
    image_count = 0

    for img_path, _ in iterate_imagestack(imagestack_path, imagestack_info):
        _, file_extension = splitext(img_path)

        if file_extension[1:] == imagestack_info['image_type']:
            image_count += 1

    if image_count == 0:
        raise Exception('Imagestack contains no valid images.')

    return



from json import load, dump

from os.path import join
from os import environ

from .const import PB_TASK

def get_task_info():
    """
    Parses the /task/task.json file and returns the parsed information as native
    python dictionary.

    Returns:

    * Task arguments (``dict``)
    """
    fp = open(join(PB_TASK, 'task.json'))
    data = load(fp)
    fp.close()
    return data


def get_petabox_info():
    """
    Parses the /task/petabox.json file and returns the parsed information as
    native python dictionary.

    Returns:

    * Petabox info (``dict``)
    """
    return load(open(join(PB_TASK, 'petabox.json')))


def write_extra_targets(extra_targets):
    """
    Create /task/extras_targets.json file based on `extra_targets`.

    Args:

    * `extra_targets`: A list of dictionaries, containing the filename as str
      in 'name', and optionally if the file is to be parsed as original in
      'mark_original', as boolean.

    Returns:

    * Nothing

    Example:

    >>> from derivermodule.files import canonical_item_filename
    >>> write_extra_targets([
    >>>     {'name': canonical_item_filename(target_file.replace('.txt', '-other.txt')),
    >>>      'mark_original': False}
    >>> ])


    """
    extra_targets_file = join(PB_TASK, 'extra_targets.json')
    fp = open(extra_targets_file, 'w+')
    dump(extra_targets, fp)
    fp.close()


def get_task_args(task_info):
    """
    Args:

    * task_info (dict): task_info as returned by get_task_info()

    Returns:

    * Tasks arguments (``dict``)
    """
    if 'task' in task_info and 'args' in task_info['task']:
        return task_info['task']['args']
    return {}


def get_task_arg_or_environ_arg(task_args, name):
    """
    Returns the value of a task argument if it is present in task_args or in
    the environment.

    The naming of task arguments is using lowercase and hypens, whereas the
    enviroment variables are upper cased and use underscores. The functions
    applies the task argument to environment variable to the `name` argument.

    So if the task argument is ``'ocr-perform-foo'``, pass that as ``name``, and
    the function will also check the environment variables for
    ``'OCR_PERFORM_FOO'``.

    Environment arguments have higher precedence over task arguments by this
    function.

    Args:

    * task_args: As returned by `get_task_args`
    * name: Custom task argument name, implementation defined.

    Returns:

    * the value of the argument, if present, and None otherwise.
    """
    environ_name = name.replace('-', '_').upper()
    if environ_name in environ:
        return environ.get(environ_name)

    if name in task_args:
        return task_args.get(name)

    return None



#: Directory of the item files within the container
PB_ITEM = '/item'
#: Directory of the task related JSON files within the container
PB_TASK = '/task'
#: Temporary (hard disk backed) temporary directory (auto cleaned upon exit)
PB_TMP = '/tmp'
#: Temporary fast (memory backed) temporary directory (auto cleaned upon exit)
#: Limited in size, currently 1.5G
PB_FAST = '/var/tmp/fast'

